@@ -10,13 +10,15 @@ import collections
 from curses import wrapper
 import time
 import os.path
+import datetime
 
 
 # Globals
 config = None
 startup_opts = None
 utterson_version = '0.02 - Dev'
-
+jekyll_config = None
+jekyll_local_server = None
 
 
 def window_prep(stdscr, title, options):
@@ -132,10 +134,10 @@ def home_screen(stdscr):
       window_prep(stdscr, "utterson", None)
       opts = collections.OrderedDict()
       opts['P'] = ['Posts', 'Created/Edit/Delete blog posts', 'normal']
-      opts['G'] = ['Pages', 'Manage static pages', 'dim']
-      opts['C'] = ['Categories', 'Created/Edit/Delete categories posts', 'dim']
-      opts['T'] = ['Tags', 'Created/Edit/Delete tags posts', 'dim']
-      opts['U'] = ['Update', 'Updates the web server', 'normal']
+      #opts['G'] = ['Pages', 'Manage static pages', 'dim']
+      #opts['C'] = ['Categories', 'Created/Edit/Delete categories posts', 'dim']
+      #opts['T'] = ['Tags', 'Created/Edit/Delete tags posts', 'dim']
+      opts['X'] = ['Tools', 'Publish Site, Local Server, etc', 'normal']
       opts['S'] = ['Settings', 'Manage utterson settings', 'normal']
       opts['Q'] = ['Quit', 'Exit utterson', 'normal']
 
@@ -147,11 +149,195 @@ def home_screen(stdscr):
       posts_main_screen(stdscr)
     elif (check_key(key, 'u')):
       update_server(stdscr)
+    elif (check_key(key, 's')):
+      setting_screen(stdscr)
+    elif (check_key(key, 'x')):
+      tools_screen(stdscr)
 
       # Prep the window.
       window_prep(stdscr, "utterson: Home", None)
       redraw = True
 
+  # Stop the jekyll server if it's running.
+  if(is_jekyll_server_running()):
+    start_stop_jekyll_server()
+
+def tools_screen(stdscr):
+  """Runs the tools screen"""
+
+  redraw = True
+  key = 0
+  notice_txt = None
+
+  # Run event loop on keys.
+  while (not check_key(key, 'q')):
+
+    if (redraw):
+
+      # Calculate menu vertical center.
+      lines_offset = (curses.LINES - 2 - 11)//2
+
+      # Calculate horizontal offset.
+      cols_offset = (curses.COLS - 45)//2
+
+      # Determine if the server is running.
+      local_server_menu_text = None
+      if( is_jekyll_server_running()):
+        local_server_menu_text = "Stop Server"
+      else:
+        local_server_menu_text = "Start Server"
+
+      # Write the main menu.
+      window_prep(stdscr, "utterson", None)
+      opts = collections.OrderedDict()
+      opts['P'] = ['Publish', 'Publishes the site to the remote server', 'normal']
+      opts['S'] = [local_server_menu_text, 'Starts and stops the local server', 'normal']
+      opts['B'] = ['Build', 'Build the static site.']
+      opts['Q'] = ['Quit', 'Exit utterson', 'normal']
+      build_full_screen_menu(stdscr, opts)
+
+      # Update notice text.
+      if (notice_txt is not None):
+        notice_header(stdscr,notice_txt)
+        notice_txt = None
+
+    key = stdscr.getch()
+
+    if (check_key(key, 's')):
+      start_stop_jekyll_server()
+      if(is_jekyll_server_running()):
+        notice_txt = 'Server running...'
+      else:
+        notice_txt = 'Server stopped'
+      redraw = True
+    elif (check_key(key, 'p')):
+      update_server(stdscr)
+    elif (check_key(key, 'b')):
+      build_jekyll_site()
+      notice_txt = 'Site Build Completed'
+      redraw = True
+
+
+
+def setting_screen(stdscr):
+  """Starts the setting screen."""
+
+  # Configure defaults
+  redraw = True
+  notice_txt = None
+  reload_settings = True
+
+  # Get access to the global config.
+  global config
+  global jekyll_config
+
+  # Begin primary key action loop.
+  key = 0
+  while (not check_key(key, 'q')):
+
+    # Rebuild the setting list if something has changed.
+    if (reload_settings):
+
+      # Reload utterson configuration file.
+      load_configuration(startup_opts['config_file'])
+      # Reload jekyll configuration file.
+      load_jekyll_configuration(config['site']['jekyll_root'] + "/_config.yml")
+
+      rows = []
+      rows.append({'return_value': 'jekyll_root',
+                   'col1': 'Jekyll Root',
+                   'col2': config['site']['jekyll_root']})
+      rows.append({'return_value': 'editing_app',
+                   'col1': 'Editing App',
+                   'col2': config['editing_app']})
+      rows.append({'return_value': 'site_name',
+                   'col1': 'Site Name',
+                   'col2': jekyll_config['name']})
+      rows.append({'return_value': 'site_description',
+                   'col1': 'Site Description',
+                   'col2': jekyll_config['description']})
+      
+      #columns = {'col1': {'title': 'Name', 'size':18}, 'col2': {'title': 'Value', 'size': 0}}
+      columns = collections.OrderedDict()
+      columns['col1'] = {'title': 'Name', 'size':18}
+      columns['col2'] = {'title': 'Value', 'size': 0}
+      il = ItemsListWindow(rows, columns)
+      il.set_window_size({'left_type': 'relative', 'left_value': 2,
+                           'right_type': 'relative', 'right_value': 2,
+                           'bottom_type': 'relative', 'bottom_value': 2,
+                           'top_type' : 'relative', 'top_value': 2,
+                           'height': 10,
+                           'width': 15})
+
+      reload_settings = False
+
+    # Redraw the whole screen if needed.
+    if (redraw):
+      window_prep(stdscr, "utterson: Settings", {'Q': 'Quit', 'E': 'Edit'})
+      if (notice_txt is not None):
+        notice_header(stdscr,notice_txt)
+        notice_txt = None
+      stdscr.refresh()
+      il.build_item_list()
+      il.refresh_window()
+      redraw = False
+
+
+    key = stdscr.getch()
+
+    if (key == curses.KEY_DOWN):
+      il.select_down()
+    elif (key == curses.KEY_UP):
+      il.select_up()
+    elif (check_key(key, 'e')):
+      selected = il.get_selected()
+      not_valid = True
+      if (selected == 'site_name'):
+        while (not_valid):
+          new_value = get_string_prompt(stdscr, 'Site Name', default_text=jekyll_config['name'])
+          if (new_value != '' and new_value is not None):
+            jekyll_config['name'] = new_value
+            save_jekyll_config(config['site']['jekyll_root'] + "/_config.yml")
+            not_valid = False
+            redraw = True
+            reload_settings = True
+            notice_txt = 'Updated: ' + selected
+      elif (selected == 'site_description'):
+        while (not_valid):
+          new_value = get_string_prompt(stdscr, 'Site Description', default_text=jekyll_config['description'])
+          if (new_value != '' and new_value is not None):
+            jekyll_config['description'] = new_value
+            save_jekyll_config(config['site']['jekyll_root'] + "/_config.yml")
+            not_valid = False
+            redraw = True
+            reload_settings = True
+            notice_txt = 'Updated: ' + selected
+      elif (selected == 'editing_app'):
+        while (not_valid):
+          new_value = get_string_prompt(stdscr, 'Editing App', default_text=config['editing_app'])
+          if (new_value != '' and new_value is not None):
+            config['editing_app'] = new_value
+            save_config(startup_opts['config_file'])
+            not_valid = False
+            redraw = True
+            reload_settings = True
+            notice_txt = 'Updated: ' + selected
+      elif (selected == 'jekyll_root'):
+        while (not_valid):
+          new_value = get_string_prompt(stdscr, 'Jekyll Root', default_text=config['site']['jekyll_root'])
+          if (new_value != '' and new_value is not None):
+            config['site']['jekyll_root'] = new_value
+            save_config(startup_opts['config_file'])
+            not_valid = False
+            redraw = True
+            reload_settings = True
+            notice_txt = 'Updated: ' + selected
+
+      # Restart the local jekyll server if the settings have been modified.
+      if(reload_settings and is_jekyll_server_running()):
+        start_stop_jekyll_server()
+        start_stop_jekyll_server()
+
 def update_server(stdscr):
   # Call the users script.
   subprocess.call(config['site']['update_cmd'])
@@ -179,12 +365,12 @@ def published_post_screen(stdscr):
           f_time_str = str(f_time.tm_year) + '-' + str(f_time.tm_mon) + '-' + str(f_time.tm_mday)
 
           temp_dict = {'return_value': f,
-                    'col1': f,
-                    'col2': f_time_str}
+                    'col2': f,
+                    'col1': f_time_str}
           posts.append(temp_dict)
 
-      posts.sort(key=lambda x: x['col2'],reverse=True)
-      columns = {'col1': {'title': 'Name', 'size':0}, 'col2': {'title': 'Modify Date', 'size': 11}}
+      posts.sort(key=lambda x: x['col1'],reverse=True)
+      columns = {'col2': {'title': 'Name', 'size':0}, 'col1': {'title': 'Modify Date', 'size': 11}}
       il = ItemsListWindow(posts, columns)
       il.set_window_size({'left_type': 'relative', 'left_value': 2,
                            'right_type': 'relative', 'right_value': 2,
@@ -196,7 +382,7 @@ def published_post_screen(stdscr):
 
     # Redraw the whole screen if needed.
     if (redraw):
-      window_prep(stdscr, "utterson:draft posts", {'Q': 'Quit', 'U': 'Unpublish', 
+      window_prep(stdscr, "utterson: Published Posts", {'Q': 'Quit', 'U': 'Unpublish', 
                                                    'I': 'Info', 'D': 'Delete',
                                                    'C': 'Change PDate'})
       if (notice_txt is not None):
@@ -237,8 +423,7 @@ def published_post_screen(stdscr):
       while (not_valid):
         date_str = get_string_prompt(stdscr, 'New Publish Date', default_text=selected[:10])
         
-        # Place holder for later regex/date verification.
-        if (True):
+        if (validate_date(date_str)):
           update_publication_date(selected, date_str)
           not_valid = False
 
@@ -281,12 +466,12 @@ def draft_post_screen(stdscr):
           f_time_str = str(f_time.tm_year) + '-' + str(f_time.tm_mon) + '-' + str(f_time.tm_mday)
 
           temp_dict = {'return_value': f,
-                    'col1': f,
-                    'col2': f_time_str}
+                    'col2': f,
+                    'col1': f_time_str}
           posts.append(temp_dict)
 
-      posts.sort(key=lambda x: x['col2'],reverse=True)
-      columns = {'col1': {'title': 'Name', 'size':0}, 'col2': {'title': 'Modify Date', 'size': 11}}
+      posts.sort(key=lambda x: x['col1'],reverse=True)
+      columns = {'col2': {'title': 'Name', 'size':0}, 'col1': {'title': 'Modify Date', 'size': 11}}
       il = ItemsListWindow(posts, columns)
       il.set_window_size({'left_type': 'relative', 'left_value': 2,
                            'right_type': 'relative', 'right_value': 2,
@@ -298,7 +483,7 @@ def draft_post_screen(stdscr):
 
     # Redraw the whole screen if needed.
     if (redraw):
-      window_prep(stdscr, "utterson:draft posts", {'Q': 'Quit', 'E': 'Edit', 'P': 'Publish', 
+      window_prep(stdscr, "utterson: Draft Posts", {'Q': 'Quit', 'E': 'Edit', 'P': 'Publish', 
                                                    'N': 'New', 'I': 'Info', 'R': 'Rename',
                                                    'D': 'Delete'})
       if (notice_txt is not None):
@@ -414,11 +599,11 @@ def deleted_post_screen(stdscr):
           f_time_str = str(f_time.tm_year) + '-' + str(f_time.tm_mon) + '-' + str(f_time.tm_mday)
 
           temp_dict = {'return_value': f,
-                    'col1': f,
-                    'col2': f_time_str}
+                    'col2': f,
+                    'col1': f_time_str}
           posts.append(temp_dict)
-      posts.sort(key=lambda x: x['col2'],reverse=True)
-      columns = {'col1': {'title': 'Name', 'size':0}, 'col2': {'title': 'Modify Date', 'size': 11}}
+      posts.sort(key=lambda x: x['col1'],reverse=True)
+      columns = {'col2': {'title': 'Name', 'size':0}, 'col1': {'title': 'Modify Date', 'size': 11}}
       il = ItemsListWindow(posts, columns)
       il.set_window_size({'left_type': 'relative', 'left_value': 2,
                            'right_type': 'relative', 'right_value': 2,
@@ -429,7 +614,7 @@ def deleted_post_screen(stdscr):
       rebuild_file_list = False
 
     if (redraw):
-      window_prep(stdscr, "utterson:draft posts", {'Q': 'Quit', 'E': 'Edit', 'R': 'Restore',
+      window_prep(stdscr, "utterson: Deleted Posts", {'Q': 'Quit', 'E': 'Edit', 'R': 'Restore',
                                                    'D': 'Delete Forever'})
       if (notice_txt is not None):
         notice_header(stdscr,notice_txt)
@@ -513,12 +698,12 @@ def template_post_screen(stdscr):
           f_time_str = str(f_time.tm_year) + '-' + str(f_time.tm_mon) + '-' + str(f_time.tm_mday)
 
           temp_dict = {'return_value': f,
-                    'col1': f,
-                    'col2': f_time_str}
+                       'col2': f,
+                       'col1': f_time_str}
           posts.append(temp_dict)
 
-      posts.sort(key=lambda x: x['col2'],reverse=True)
-      columns = {'col1': {'title': 'Name', 'size':0}, 'col2': {'title': 'Modify Date', 'size': 11}}
+      posts.sort(key=lambda x: x['col1'],reverse=True)
+      columns = {'col2': {'title': 'Name', 'size':0}, 'col1': {'title': 'Modify Date', 'size': 11}}
       il = ItemsListWindow(posts, columns)
       il.set_window_size({'left_type': 'relative', 'left_value': 2,
                            'right_type': 'relative', 'right_value': 2,
@@ -530,7 +715,7 @@ def template_post_screen(stdscr):
 
     # Redraw the whole screen if needed.
     if (redraw):
-      window_prep(stdscr, "utterson:draft posts", {'Q': 'Quit', 'E': 'Edit', 
+      window_prep(stdscr, "utterson: Templates", {'Q': 'Quit', 'E': 'Edit', 
                                                    'N': 'New', 'R': 'Rename', 'D': 'Delete'})
       if (notice_txt is not None):
         notice_header(stdscr,notice_txt)
@@ -642,6 +827,12 @@ def load_configuration(config_file_dir):
   config = yaml.load(stream)
   stream.close()
 
+def load_jekyll_configuration(config_file_path):
+  stream = open(config_file_path, 'r')
+  global jekyll_config
+  jekyll_config = yaml.load(stream)
+  stream.close()
+
 def yes_no_prompt(stdscr, question):
   lines_offset = (curses.LINES - 3) // 2
   cols_offset = (curses.COLS - len(question) - 2) // 2
@@ -670,8 +861,9 @@ def post_info_window(stdscr, post_path):
 
   window.addstr(1,1,('Title: ' + post.meta_data['title'])[:61])
   window.addstr(2,1,'Summary:')
-  window.addstr(3,1,' ' + post.meta_data['summary'][:66])
-  window.addstr(4,1,' ' + post.meta_data['summary'][66:132])
+  if post.meta_data['summary'] is not None:
+    window.addstr(3,1,' ' + post.meta_data['summary'][:66])
+    window.addstr(4,1,' ' + post.meta_data['summary'][66:132])
   window.addstr(5,1,'Categories:' + str(post.meta_data['categories']))
   window.addstr(6,1,'Tags: ' + str(post.meta_data['tags']))
   window.refresh()
@@ -732,6 +924,18 @@ def generate_jekyll_config(path, site_title):
   cf.write('paginate: 7\n')
   cf.close()
 
+def save_config(config_file_path):
+  global config
+  config_file = open(config_file_path, 'w')
+  config_file.write(yaml.dump(config, default_flow_style=False))
+  config_file.close()
+
+def save_jekyll_config(config_file_path):
+  global jekyll_config
+  jekyll_config_file = open(config_file_path, 'w')
+  jekyll_config_file.write(yaml.dump(jekyll_config, default_flow_style=False))
+  jekyll_config_file.close()
+
 def build_new_utterson_site(root_folder_name):
   """Builds an empty utterson site based on jekyll."""
 
@@ -814,20 +1018,60 @@ def edit_post(path):
   else:
     subprocess.call(['vim', path])
 
+def validate_date(date_text):
+  try:
+    datetime.datetime.strptime(date_text, '%Y-%m-%d')
+    return True
+  except ValueError:
+    return False
+
+  return False
+
 def update_publication_date(file_name, date_str):
   """Edits the publication date if it's changed."""
 
-  # Verify date string. TODO
-  post = JekyllPost(config['site']['jekyll_root'] + "/_posts/" + file_name)
-  post.meta_data['date_published'] = date_str
-  post.meta_data['date_published'] = date_str
-  file_short_name = file_name[11:]
-  post_file_path = config['site']['jekyll_root'] + "/_posts/" + \
-                   date_str + '-' + file_name[11:]
-  post.save_post(post_file_path)
-  os.remove(config['site']['jekyll_root'] + "/_posts/" + file_name)
+  if(validate_date(date_str)):
+    post = JekyllPost(config['site']['jekyll_root'] + "/_posts/" + file_name)
+    post.meta_data['date_published'] = date_str
+    post.meta_data['date_published'] = date_str
+    file_short_name = file_name[11:]
+    post_file_path = config['site']['jekyll_root'] + "/_posts/" + \
+                     date_str + '-' + file_name[11:]
+    post.save_post(post_file_path)
+    os.remove(config['site']['jekyll_root'] + "/_posts/" + file_name)
+    return True
+  else:
+    return False
 
 
+def build_jekyll_site():
+  site_root = config['site']['jekyll_root']
+  deploy_root = config['site']['jekyll_root'] + '/_site'
+  subprocess.call(['jekyll', 'build', '-s', site_root, '-d', deploy_root])
+
+
+def start_stop_jekyll_server():
+  """Stars or stops the jekyll server in watch mode."""
+  global jekyll_local_server
+
+  # If it's running, kill it.
+  if (is_jekyll_server_running()):
+    jekyll_local_server.terminate()
+    jekyll_local_server.wait()
+    jekyll_local_server = None
+  else:
+    site_root = config['site']['jekyll_root']
+    deploy_root = config['site']['jekyll_root'] + '/_site'
+    jekyll_local_server = subprocess.Popen(['jekyll', 'server', '--watch', '-s', site_root, '-d', deploy_root], 
+                                           stdout=subprocess.PIPE, stderr=subprocess.PIPE, stdin=subprocess.PIPE)
+
+def is_jekyll_server_running():
+  """Checks if the jekyll server is running. If so returns true."""
+
+  if ((jekyll_local_server is not None) and (jekyll_local_server.poll() is None)):
+    return True
+
+  return False
 
 
 class JekyllPost:
@@ -1068,8 +1312,11 @@ class ItemsListWindow:
       shared_col_size = (self.window_width - used_col_space) // shared_col_space
 
       cur_col = 0
-      for key,value in self.columns.items():
+      for column in range(1,len(self.columns.items())+1,1):
+        key = "col" + str(column)
+      #for key,value in self.columns.items():
         # Get col size
+        value = self.columns[key]
         if (value['size'] == 0):
           cur_col_size = shared_col_size
         else:
@@ -1092,17 +1339,31 @@ class ItemsListWindow:
     for item_number in range(self.last_top_line, self.last_top_line + self.get_visible_lines()):
       if item_number == self.selected_line:
         col_num = 0
-        for key, value in self.items[item_number].items():
-          if (key != 'return_value'):
-            self.window.addstr(line_number, col_num, self.white_space_pad(self.items[item_number][key], header_values[key]), curses.A_STANDOUT)
-            col_num += header_values[key]
+        for colNum in range(1,len(self.items[item_number].items()),1):
+          key = "col" + str(colNum)
+          self.window.addstr(line_number, col_num, self.white_space_pad(self.items[item_number][key], header_values[key]), curses.A_STANDOUT)
+          col_num += header_values[key]
+
+        #col_num = 0
+        #for key, value in self.items[item_number].items():
+        #  if (key != 'return_value'):
+        #    self.window.addstr(line_number, col_num, self.white_space_pad(self.items[item_number][key], header_values[key]), curses.A_STANDOUT)
+        #    col_num += header_values[key]
+
         #self.window.addstr(line_number, 0, self.items[item_number]['col2'], curses.A_STANDOUT)
       else:
         col_num = 0
-        for key, value in self.items[item_number].items():
-          if (key != 'return_value'):
-            self.window.addstr(line_number, col_num, self.white_space_pad(self.items[item_number][key], header_values[key]))
-            col_num += header_values[key]
+        for colNum in range(1,len(self.items[item_number].items()),1):
+          key = "col" + str(colNum)
+          self.window.addstr(line_number, col_num, self.white_space_pad(self.items[item_number][key], header_values[key]))
+          col_num += header_values[key]
+
+        #col_num = 0
+        #for key, value in self.items[item_number].items():
+        #  if (key != 'return_value'):
+        #    self.window.addstr(line_number, col_num, self.white_space_pad(self.items[item_number][key], header_values[key]))
+        #    col_num += header_values[key]
+
       line_number += 1
 
   def white_space_pad(self, input_str, length):
